Attribute VB_Name = "modMCI"
' Modulname : vbzMCI.bas
' Version   : 1.1
' Stand     : 31.12.2004
' Quelle    : http://www.vb-zentrum.de
' Autor     : Ralf Schlegel

Option Explicit

' Globale Variablen:
Public mciFileName As String
Public mciTotalTime As String
Public mciTotalSec As Long

' Lokale Variablen
Private mciPauseFlag As Boolean
Private mciOpenDoor As Boolean

Private Declare Function GetShortPathName Lib "kernel32" Alias _
        "GetShortPathNameA" (ByVal lpszLongPath As String, ByVal _
        lpszShortPath As String, ByVal cchBuffer As Long) As Long
Private Declare Function mciSendString Lib "winmm.dll" Alias _
        "mciSendStringA" (ByVal lpstrCommand As String, ByVal _
        lpstrReturnString As String, ByVal uReturnLength As Long, _
        ByVal hwndCallback As Long) As Long

' Vereifachte Form der DLL-Funktion 'mciSendString':
' Erwartet einen String und gibt ggf. einen Long-Wert zurck
Private Function SendMCI(Com As String) As Long
  Dim Tmp As String * 256
  
  mciSendString Com, Tmp, 256, 0
  SendMCI = Val(Tmp)
End Function

' Art der Multimediadatei ber Extension ermitteln:
' Erwartet einen Dateinamen, gibt den MCI-Typen zurck
Private Function mciType(fname As String) As String
  Select Case LCase(Mid$(fname, InStrRev(fname, ".") + 1))
    Case "cda"
      mciType = "CDAudio"
    Case "mid", "midi", "rmi"
      mciType = "Sequencer"
    Case "wav"
      mciType = "WaveAudio"
    Case "avi"
      mciType = "AVIVideo"
    Case "mp3", "snd", "aif", "aifc", "aiff", "au"
      mciType = "MPEGVideo"
    Case "wma", "wmv"
      mciType = "MPEGVideo2"
    Case "mpeg", "mpg", "m1v", "mp2", "mpa", "mpe"
      mciType = "MPEGVideo"
    Case Else
      mciType = ""
  End Select
End Function
 
' wandelt die bergebenen Millisekunden in einen
' anzeigbaren String 'Min:Sek', also '00:00' um
Private Function mciTimeString(nMSec As Long) As String
  Dim nMin As Integer
  Dim nSec As Integer

  nSec = nMSec \ 1000
  nMin = nMSec \ 60000
  nSec = nSec - (nMin * 60)
  mciTimeString = Format(nMin, "00\:") & Format(nSec, "00")
End Function

' Aktuelle Abspielposition des Mediafiles:
Public Function mciTimePos() As String
  Dim pos As Long
  
  On Error Resume Next
  pos = SendMCI("Status mciAlias Position")
  mciTimePos = mciTimeString(pos)
End Function

' Ermittelt die Position eines CD-Audio Tracks:
Private Function getCDTrackPos(track As Long) As Long
  getCDTrackPos = SendMCI("Status mciAlias Position Track " & track)
End Function

' Spielt das mit 'track' bergebene CD-Audiostck ab:
Public Sub PlayTrack(track As Long)
  SendMCI "Play mciAlias From " & getCDTrackPos(track)
End Sub

' Gibt TRUE zurck, wenn in dem mit 'drv' abgefragten Laufwerk
' ein Medium eingelegt ist (das kann auch eine CD-ROM sein!)
Public Function mciCDpresent(ByVal drv As String) As Boolean
  Dim strAlias As String
  
  strAlias = "Laufwerk" & Left$(drv, 1)
  SendMCI "Open " & Left$(drv, 1) & ": type CDAudio alias " & strAlias
  mciCDpresent = CBool(SendMCI("Status " & strAlias & " Media Present"))
  SendMCI "Close " & strAlias
End Function

' ffnet oder Schliet das bergebene Laufwerk:
Public Sub mciEjectCD(ByVal drv As String)
  Dim strAlias As String
  
  strAlias = "Laufwerk" & Left$(drv, 1)
  SendMCI "Open " & Left$(drv, 1) & ": type CDAudio alias " & strAlias
  If mciOpenDoor Then
    SendMCI "Set " & strAlias & " Door Closed"
  Else
    SendMCI "Set " & strAlias & " Door Open"
  End If
  SendMCI "Close " & strAlias
  mciOpenDoor = Not mciOpenDoor
End Sub

' Spielt die mit 'fname' als kompletten Pfad bergebene
' Mediadatei ab, optional kann die Startposition in Sekunden bergeben werden:
Public Function mciPlay(fname As String, Optional SecPos As Long = 0) As Boolean
  Dim l As Long
  Dim Tmp As String * 256
  Dim mciString As String
  
  mciString = mciType(fname)        ' verwendbare Typen ermitteln
  If mciString = "" Then Exit Function ' wenn nicht, dann Ende
  l = GetShortPathName(fname, Tmp, 256)
  If l <> 0 Then
    mciStop                         ' vorheriges Stck beenden
    mciString = " type " & mciString
    SendMCI "Open " & Left$(Tmp, l) & mciString & " alias mciAlias"
    SendMCI "Set mciAlias time format milliseconds"
    ' globale Variablen:
    mciTotalSec = SendMCI("status mciAlias length")
    mciTotalTime = mciTimeString(mciTotalSec)
    mciTotalSec = mciTotalSec \ 1000
    mciFileName = Mid$(fname, InStrRev(fname, "\") + 1)
    SendMCI "Play mciAlias From " & SecPos * 1000
  End If
  mciPlay = True
End Function

' Unterbricht das Abspielen der aktuellen
' Mediadatei, bzw. setzt sie wieder fort:
Public Sub mciPause()
  mciPauseFlag = Not mciPauseFlag
  If mciPauseFlag Then
    SendMCI "Pause mciAlias"
  Else
    SendMCI "Resume mciAlias"
    SendMCI "Put mciAlias Destination"
  End If
End Sub

' Beendet das Abspielen und schliet die
' aktuelle Mediadatei:
Public Sub mciStop()
  mciPauseFlag = False
  SendMCI "Stop mciAlias"
  SendMCI "Close mciAlias"
  mciTotalTime = ""
  mciFileName = ""
End Sub
